#loader-container {
    width: 100vw;
    height: 100vh;
    position: absolute;
    margin: 0;
    padding: 0;
    z-index: 998;
    display: flex;
    justify-content: center;
    align-items: center;
}
