"""
# Card

"""

import solara
import solara.lab
from solara.website.components import NoPage
from solara.website.utils import apidoc

title = "Card"

Page = NoPage

__doc__ += apidoc(solara.Card.f)  # type: ignore
